import streamlit as st
from aiyt.ui import app_header, caption_ui, chat_ui, divider, input_ui, transcribe_ui
from pathlib import Path
from pytubefix import YouTube


def body():
    with st.container(border=True, key="main-container"):
        api_key, model, url = input_ui()

        try:
            yt = YouTube(url)
            yt.check_availability()
        except Exception as e:
            st.error(e)
            st.stop()

        langs = [c.code for c in yt.captions]

        divider(key=1)

        if langs or not yt:
            transcript = caption_ui(yt, langs, api_key, model)
        else:
            transcript = transcribe_ui(yt, api_key, model)

    if transcript:
        chat_ui(transcript, api_key, model)


def app():
    st.html(Path(__file__).parent / "style.css")
    app_header(icon="youtube_activity", color="red")
    st.write("")
    body()


if __name__ == "__main__":
    app()



import os
import streamlit as st
from aiyt.utils import add_punctuation, consolidate_messages, transcribe
from google.genai import Client, types
from pytubefix import YouTube
from textwrap import dedent

MODELS = [
    "gemini-2.5-flash-lite",
    "gemini-2.5-flash",
    "gemini-2.5-pro",
]
sess = st.session_state


def app_header(icon: str, color: str):
    icon_with_color = f":{color}[:material/{icon}:]"
    with st.container(key="app-header"):
        st.markdown(f"## {icon_with_color} &nbsp; aiyt")
        st.caption("Transcribe, Chat and Summarize Youtube Video with AI")


def divider(key: int = 1):
    with st.container(key=f"divider{key}"):
        st.divider()


def input_ui() -> tuple[str, str, str]:
    """Streamlit UI for inputting API key, model, and YouTube URL"""
    with st.form(key="input-form", enter_to_submit=True, border=False):
        c1, c2 = st.columns(2)
        api_key = c1.text_input(
            "Gemini API key",
            key="google-api-key",
            type="password",
            value=os.getenv("GOOGLE_API_KEY", ""),
            help="Visit [gemini docs](https://ai.google.dev/gemini-api/docs/api-key) to get the API key",
        )
        model = c2.selectbox(
            "Select the model",
            key="model",
            options=MODELS,
            index=0,
        )

        c1, c2 = st.columns([3, 1], vertical_alignment="bottom")
        url = c1.text_input("Youtube URL", key="url-input")

        c2.form_submit_button(
            "Submit",
            use_container_width=True,
            on_click=lambda: sess.update({"chat": None}),
        )

        submittable = api_key and model and url
        if not submittable:
            st.stop()

    return api_key, model, url


def caption_ui(yt: YouTube | None, langs: list[str], api_key: str, model: str) -> None:
    st.markdown("#### 💬 &nbsp; Extract Captions")

    st.selectbox(
        label="Select the language",
        key="caption_lang",
        options=langs,
        index=None,
        format_func=lambda x: x.split(".")[-1],
    )

    st.radio(
        label="Select the format",
        key="caption_format",
        options=["srt", "txt", "ai formatted"],
        index=0,
        horizontal=True,
        disabled=not sess.caption_lang,
    )

    transcript = ""
    if sess.caption_lang:
        if sess.caption_format == "srt":
            transcript = yt.captions[sess.caption_lang].generate_srt_captions()
        else:
            raw_transcript = yt.captions[sess.caption_lang].generate_txt_captions()
            if sess.caption_format == "txt":
                transcript = raw_transcript
            elif sess.caption_format == "ai formatted":
                transcript = add_punctuation(api_key, raw_transcript, model)

    sess.caption_output = transcript
    st.text_area(
        label="Captions",
        key="caption_output",
        height=400,
        disabled=not transcript,
    )

    return transcript


def transcribe_ui(yt: YouTube, api_key: str, model: str) -> str:
    """Streamlit UI for transcribing audio"""
    id = yt.video_id
    st.markdown("#### 🗣️ &nbsp; Transcribe Audio")
    if "transcribe_consent" not in sess:
        sess.transcribe_consent = dict()
    consent = st.empty()
    with consent.container():
        if not sess.transcribe_consent.setdefault(id, False):
            st.info("No captions found, transcribe audio with Gemini?")
            if st.button("Transcribe"):
                sess.transcribe_consent[id] = True
    if sess.transcribe_consent[id]:
        consent.empty()
        with st.spinner("Transcribing audio...", show_time=True):
            st.text_area(
                label="Transcript",
                value=transcribe(id, model, api_key),
                key="transcript_output",
                height=400,
            )
            return sess.transcript_output


def chat_ui(transcript: str, api_key: str, model: str) -> None:
    """Streamlit chat interface for interacting with the transcript"""
    divider(key=2)

    sys_prompt = dedent(f"""\
            You are a helpful assistant that can answer questions about this transcript:
            <transcript>
            {transcript}
            </transcript>
        """)

    # Initialize chat object in session state
    if sess.chat is None:
        client = Client(api_key=api_key)
        sess.chat = client.chats.create(
            model=model,
            config=types.GenerateContentConfig(system_instruction=sys_prompt),
        )

    # Display chat history with consolidated messages
    avatar = {"user": "🐒", "model": "💭"}
    consolidated_messages = consolidate_messages(sess.chat.get_history())
    for role, text in consolidated_messages:
        with st.chat_message(role, avatar=avatar[role]):
            st.markdown(text)

    # Accept user input
    if prompt := st.chat_input("chat about the transcript..."):
        # Display user message
        with st.chat_message("user", avatar=avatar["user"]):
            st.markdown(prompt)

        # Generate and display assistant response
        with st.chat_message("model", avatar=avatar["model"]):
            try:
                response = sess.chat.send_message_stream(prompt)
                st.write_stream(chunk.text for chunk in response)

            except Exception as e:
                st.error(e)
                st.stop()



import click
import sys
from pathlib import Path
from subprocess import Popen


@click.command()
@click.version_option()
def main():
    """Launch the Streamlit app."""

    cmd = f"{sys.executable} -m streamlit run {Path(__file__).parent / 'main.py'}"
    process = Popen(cmd.split())
    try:
        process.wait()
    except KeyboardInterrupt:
        process.terminate()
        exit(0)


if __name__ == "__main__":
    main()



import streamlit as st
from google.genai import Client, types
from io import BytesIO
from pytubefix import Buffer, YouTube

sess = st.session_state


@st.cache_data(show_spinner=False)
def add_punctuation(api_key: str, transcript: str, model: str) -> str:
    """Add punctuation to a transcript using Gemini's LLM."""
    sys_prompt = "add punctuations and appropiate paragraphs to the following text, do not add any comments"
    client = Client(api_key=api_key)
    response = client.models.generate_content(
        model=model,
        config=types.GenerateContentConfig(system_instruction=sys_prompt),
        contents=transcript,
    )
    return response.text


def download_audio_from_yt(id: str) -> tuple[Buffer, str]:
    """download the lowest quality audio stream to a pytubefix Buffer object"""
    yt = YouTube(f"https://youtu.be/{id}")
    audio_streams = yt.streams.filter(only_audio=True, audio_codec="opus")
    lowest_quality_stream = audio_streams.order_by("abr").first()
    if not lowest_quality_stream:
        st.error("No audio stream found")
        st.stop()
    buffer = Buffer()
    buffer.download_in_buffer(lowest_quality_stream)
    mime_type = lowest_quality_stream.mime_type
    return buffer, mime_type


# reserve for one-off upload use cases
# def get_audio_part(mime_type: str, buffer: Buffer) -> types.Part:
#     """convert the pytubefix Buffer object to a Gemini types.Part object"""
#     return types.Part.from_bytes(data=buffer.read(), mime_type=mime_type)


def remove_duplicate_gemini_audio(name: str, client: Client):
    """adding audio with the same name to Gemini cloud storage will raise an error, so remove the existing one"""
    audio_files = [f.name for f in client.files.list()]
    for file_name in audio_files:
        if name in file_name:
            client.files.delete(name=file_name)


def upload_audio_to_gemini(
    name: str, buffer: Buffer, mime_type: str, client: Client
) -> types.File:
    """upload the audio to Gemini cloud storage"""
    io_obj: BytesIO = buffer.buffer
    io_obj.seek(0)
    upload_config = types.UploadFileConfig(mime_type=mime_type, name=name)
    remove_duplicate_gemini_audio(name, client)
    return client.files.upload(file=io_obj, config=upload_config)


@st.cache_data(show_spinner=False)
def transcribe(
    id: str,
    model: str,
    api_key: str,
    system_prompt: str = "You are a professional transcriber. You output only transcript, no other text.",
    user_prompt: str = "Generate a transcript of the speech",
) -> str:
    """transcribe the audio using Gemini"""
    filename = id.lower().replace("_", "-")
    buffer, mime_type = download_audio_from_yt(id)
    client = Client(api_key=api_key)
    audio_file = upload_audio_to_gemini(filename, buffer, mime_type, client)

    response = client.models.generate_content(
        model=model,
        config=types.GenerateContentConfig(system_instruction=system_prompt),
        contents=[user_prompt, audio_file],
    )
    return response.text


def consolidate_messages(messages: list) -> list:
    """Consolidate consecutive messages with the same role into single messages"""
    if not messages:
        return []

    consolidated = []
    current_role = None
    current_text = ""

    for message in messages:
        if current_role is None:
            current_role = message.role
            current_text = message.parts[0].text
        elif message.role == current_role:
            # Same role, concatenate text
            current_text += message.parts[0].text
        else:
            # Different role, save current and start new
            consolidated.append((current_role, current_text))
            current_role = message.role
            current_text = message.parts[0].text

    # Don't forget the last message
    if current_role is not None:
        consolidated.append((current_role, current_text))

    return consolidated
